@@ -278,7 +278,7 @@ tile_surface (cairo_surface_t *surface,
         gtk_style_context_add_provider (context,
                                         GTK_STYLE_PROVIDER (provider),
                                         GTK_STYLE_PROVIDER_PRIORITY_THEME);
-        gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, &bg);
+        gtk_style_context_get (context, GTK_STATE_FLAG_NORMAL, "background-color", &bg, NULL);
         gdk_cairo_set_source_rgba(cr, &bg);
         g_object_unref (G_OBJECT (context));
     }
@@ -326,6 +326,18 @@ mate_bg_crossfade_set_start_surface (MateBGCrossfade* fade, cairo_surface_t *sur
     return fade->priv->start_surface != NULL;
 }
 
+#if GLIB_CHECK_VERSION(2,61,2)
+static gdouble
+get_current_time (void)
+{
+    const double microseconds_per_second = (double) G_USEC_PER_SEC;
+    gint64 tv;
+
+    tv = g_get_real_time ();
+
+    return (double) (tv / microseconds_per_second);
+}
+#else
 static gdouble
 get_current_time (void)
 {
@@ -340,6 +352,7 @@ get_current_time (void)
 
     return timestamp;
 }
+#endif
 
 /**
  * mate_bg_crossfade_set_end_surface:
