@@ -75,10 +75,6 @@ struct _FileSize {
 	char* file;
 };
 
-/* This is the size of the GdkRGB dither matrix, in order to avoid
- * bad dithering when tiling the gradient
- */
-#define GRADIENT_PIXMAP_TILE_SIZE 128
 #define THUMBNAIL_SIZE 256
 
 typedef struct FileCacheEntry FileCacheEntry;
@@ -104,7 +100,7 @@ struct _MateBG {
 
 	/* Cached information, only access through cache accessor functions */
 	SlideShow* slideshow;
-	time_t file_mtime;
+	gint64 file_mtime;
 	GdkPixbuf* pixbuf_cache;
 	int timeout_id;
 
@@ -172,7 +168,7 @@ static GdkPixbuf *get_pixbuf_for_size  (MateBG               *bg,
 static void       clear_cache          (MateBG               *bg);
 static gboolean   is_different         (MateBG               *bg,
 					const char            *filename);
-static time_t     get_mtime            (const char            *filename);
+static gint64     get_mtime            (const char            *filename);
 static GdkPixbuf *create_img_thumbnail (MateBG               *bg,
 					MateDesktopThumbnailFactory *factory,
 					GdkScreen             *screen,
@@ -441,7 +437,6 @@ mate_bg_save_to_gsettings (MateBG    *bg,
 	g_free (secondary);
 }
 
-
 static void
 mate_bg_init (MateBG *bg)
 {
@@ -685,16 +680,10 @@ static gboolean
 cache_file_is_valid (const char *filename,
 		     const char *cache_filename)
 {
-	time_t mtime;
-	time_t cache_mtime;
-
 	if (!g_file_test (cache_filename, G_FILE_TEST_IS_REGULAR))
 		return FALSE;
 
-	mtime = get_mtime (filename);
-	cache_mtime = get_mtime (cache_filename);
-
-	return (mtime < cache_mtime);
+	return (get_mtime (filename) < get_mtime (cache_filename));
 }
 
 static void
@@ -940,7 +929,6 @@ get_scaled_pixbuf (MateBGPlacement  placement,
 	return new;
 }
 
-
 static void
 draw_image_area (MateBG        *bg,
 		 gint           num_monitor,
@@ -1175,6 +1163,7 @@ mate_bg_create_surface_scale (MateBG      *bg,
 
 	cairo_surface_t *surface;
 	cairo_t *cr;
+	GdkDisplay *display;
 
 	g_return_val_if_fail (bg != NULL, NULL);
 	g_return_val_if_fail (window != NULL, NULL);
@@ -1189,7 +1178,9 @@ mate_bg_create_surface_scale (MateBG      *bg,
 
 	mate_bg_get_pixmap_size (bg, width, height, &pm_width, &pm_height);
 
-	if (root)
+	display = gdk_display_get_default ();
+
+	if ((root) &&  GDK_IS_X11_DISPLAY (display))
 	{
 		surface = make_root_pixmap (window, pm_width * scale, pm_height * scale);
 	}
@@ -1223,7 +1214,6 @@ mate_bg_create_surface_scale (MateBG      *bg,
 	return surface;
 }
 
-
 /* determine if a background is darker or lighter than average, to help
  * clients know what colors to draw on top with
  */
@@ -1703,7 +1693,6 @@ struct _SlideShow
 	GQueue *stack;
 };
 
-
 #if GLIB_CHECK_VERSION(2,61,2)
 static double
 now (void)
@@ -2049,7 +2038,6 @@ blow_expensive_caches_in_idle (MateBG *bg)
 	}
 }
 
-
 static gboolean
 on_timeout (gpointer data)
 {
@@ -2103,14 +2091,12 @@ ensure_timeout (MateBG *bg,
 	}
 }
 
-static time_t
+static gint64
 get_mtime (const char *filename)
 {
 	GFile     *file;
 	GFileInfo *info;
-	time_t     mtime;
-
-	mtime = (time_t)-1;
+	gint64     mtime = (gint64)-1;
 
 	if (filename) {
 		file = g_file_new_for_path (filename);
@@ -2324,7 +2310,7 @@ find_best_size (GSList *sizes, gint width, gint height)
 				best = size;
 			}
 			else if (d == distance) {
-				if (abs (size->width - width) < abs (best->width - width)) {
+				if (best && (abs (size->width - width) < abs (best->width - width))) {
 					best = size;
 				}
 			}
@@ -2438,9 +2424,7 @@ is_different (MateBG    *bg,
 		return FALSE;
 	}
 	else {
-		time_t mtime = get_mtime (filename);
-
-		if (mtime != bg->file_mtime)
+		if (get_mtime (filename) != bg->file_mtime)
 			return TRUE;
 
 		if (strcmp (filename, bg->filename) != 0)
@@ -3060,7 +3044,6 @@ read_slideshow_file (const char *filename,
 		show = NULL;
 	}
 
-
 	if (show) {
 		if (!g_markup_parse_context_end_parse (context, err)) {
 			slideshow_unref (show);
@@ -3103,13 +3086,12 @@ create_thumbnail_for_filename (MateDesktopThumbnailFactory *factory,
 			       const char            *filename)
 {
 	char *thumb;
-	time_t mtime;
+	gint64 mtime;
 	GdkPixbuf *orig, *result = NULL;
 	char *uri;
 
 	mtime = get_mtime (filename);
-
-	if (mtime == (time_t)-1)
+	if (mtime == (gint64)-1)
 		return NULL;
 
 	uri = g_filename_to_uri (filename, NULL, NULL);
@@ -3131,7 +3113,6 @@ create_thumbnail_for_filename (MateDesktopThumbnailFactory *factory,
 
 			result = pixbuf_scale_to_fit (orig, THUMBNAIL_SIZE, THUMBNAIL_SIZE);
 
-
 			g_object_set_data_full (G_OBJECT (result), "mate-thumbnail-height",
 						g_strdup_printf ("%d", orig_height), g_free);
 			g_object_set_data_full (G_OBJECT (result), "mate-thumbnail-width",
@@ -3239,8 +3220,7 @@ mate_bg_create_frame_thumbnail (MateBG			*bg,
 	if (!show)
 		return NULL;
 
-
-	if (frame_num < 0 || frame_num >= g_queue_get_length (show->slides))
+	if (frame_num < 0 || (guint) frame_num >= g_queue_get_length (show->slides))
 		return NULL;
 
 	i = 0;
@@ -3261,7 +3241,6 @@ mate_bg_create_frame_thumbnail (MateBG			*bg,
 	if (!found)
 		return NULL;
 
-
 	result = gdk_pixbuf_new (GDK_COLORSPACE_RGB, FALSE, 8, dest_width, dest_height);
 
 	draw_color (bg, result);
