@@ -52,8 +52,6 @@
 #include <gtk/gtk.h>
 #endif
 
-#define sure_string(s) ((s)!=NULL?(s):"")
-
 #define MATE_DESKTOP_USE_UNSTABLE_API
 #undef MATE_DISABLE_DEPRECATED
 #include <mate-desktop-item.h>
@@ -84,14 +82,14 @@ struct _MateDesktopItem {
 
 	char *location;
 
-	time_t mtime;
+	gint64 mtime;
 
 	guint32 launch_time;
 };
 
 /* If mtime is set to this, set_location won't update mtime,
  * this is to be used internally only. */
-#define DONT_UPDATE_MTIME ((time_t)-2)
+#define DONT_UPDATE_MTIME ((gint64)-2)
 
 typedef struct {
 	char *name;
@@ -414,7 +412,6 @@ copy_string_hash (gpointer key, gpointer value, gpointer user_data)
 			      g_strdup (value));
 }
 
-
 /**
  * mate_desktop_item_copy:
  * @item: The item to be copied
@@ -509,7 +506,6 @@ make_fake_directory (GFile *dir)
 	mate_desktop_item_set_entry_type (item,
 					   MATE_DESKTOP_ITEM_TYPE_DIRECTORY);
 
-
 	item->mtime = DONT_UPDATE_MTIME; /* it doesn't exist, we know that */
 	child = g_file_get_child (dir, ".directory");
 	mate_desktop_item_set_location_gfile (item, child);
@@ -583,7 +579,7 @@ mate_desktop_item_new_from_gfile (GFile *file,
 	GFileInfo *info;
 	GFileType type;
 	GFile *parent;
-	time_t mtime = 0;
+	gint64 mtime = 0;
 	ReadBuf *rb;
 
 	g_return_val_if_fail (file != NULL, NULL);
@@ -842,7 +838,7 @@ mate_desktop_item_save (MateDesktopItem *item,
 		return FALSE;
 
 	item->modified = FALSE;
-	item->mtime = time (NULL);
+	item->mtime = g_get_real_time () / G_USEC_PER_SEC;
 
 	return TRUE;
 }
@@ -976,7 +972,6 @@ key_basename (const char *key)
 		return key;
 }
 
-
 static const char *
 lookup (const MateDesktopItem *item, const char *key)
 {
@@ -1464,6 +1459,7 @@ expand_string (const MateDesktopItem  *item,
 	return g_string_free (gs, FALSE);
 }
 
+/*Functions in this code block should only ever be reached in x11*/
 #ifdef HAVE_STARTUP_NOTIFICATION
 static void
 sn_error_trap_push (SnDisplay *display,
@@ -1662,7 +1658,7 @@ add_startup_timeout (GdkScreen         *screen,
 						data);
 	}
 }
-#endif /* HAVE_STARTUP_NOTIFICATION */
+#endif /* HAVE_STARTUP_NOTIFICATION - functions should only be reached in x11*/
 
 static inline char *
 stringify_uris (GSList *args)
@@ -1815,64 +1811,71 @@ ditem_execute (const MateDesktopItem *item,
 	arg_ptr = make_args (file_list);
 
 #ifdef HAVE_STARTUP_NOTIFICATION
-	if (screen)
-		gdkdisplay = gdk_screen_get_display (screen);
-	else
-		gdkdisplay = gdk_display_get_default ();
+	GdkDisplay *display = gdk_screen_get_display (gdk_screen_get_default());
+	if (GDK_IS_X11_DISPLAY (display))
+	{
+		if (screen)
+			gdkdisplay = gdk_screen_get_display (screen);
+		else
+			gdkdisplay = gdk_display_get_default ();
 
-	sn_display = sn_display_new (GDK_DISPLAY_XDISPLAY (gdkdisplay),
-				     sn_error_trap_push,
-				     sn_error_trap_pop);
+		sn_display = sn_display_new (GDK_DISPLAY_XDISPLAY (gdkdisplay),
+					     sn_error_trap_push,
+					     sn_error_trap_pop);
 
+		/* Only initiate notification if desktop file supports it.
+		 * (we could avoid setting up the SnLauncherContext if we aren't going
+		 * to initiate, but why bother)
+		 */
 
-	/* Only initiate notification if desktop file supports it.
-	 * (we could avoid setting up the SnLauncherContext if we aren't going
-	 * to initiate, but why bother)
-	 */
+		startup_class = mate_desktop_item_get_string (item,
+							      "StartupWMClass");
+		if (startup_class ||
+		    mate_desktop_item_get_boolean (item, "StartupNotify")) {
+			const char *name;
+			const char *icon;
 
-	startup_class = mate_desktop_item_get_string (item,
-						       "StartupWMClass");
-	if (startup_class ||
-	    mate_desktop_item_get_boolean (item, "StartupNotify")) {
-		const char *name;
-		const char *icon;
+			sn_context = sn_launcher_context_new (sn_display,
+							      screen ? gdk_x11_screen_get_screen_number (screen) :
+							      DefaultScreen (GDK_DISPLAY_XDISPLAY (gdkdisplay)));
 
-		sn_context = sn_launcher_context_new (sn_display,
-						      screen ? gdk_x11_screen_get_screen_number (screen) :
-						      DefaultScreen (GDK_DISPLAY_XDISPLAY (gdkdisplay)));
-
-		name = mate_desktop_item_get_localestring (item,
-							    MATE_DESKTOP_ITEM_NAME);
-
-		if (name == NULL)
 			name = mate_desktop_item_get_localestring (item,
-								    MATE_DESKTOP_ITEM_GENERIC_NAME);
+								   MATE_DESKTOP_ITEM_NAME);
 
-		if (name != NULL) {
-			char *description;
+			if (name == NULL)
+				name = mate_desktop_item_get_localestring (item,
+									   MATE_DESKTOP_ITEM_GENERIC_NAME);
 
-			sn_launcher_context_set_name (sn_context, name);
+			if (name != NULL) {
+				char *description;
 
-			description = g_strdup_printf (_("Starting %s"), name);
+				sn_launcher_context_set_name (sn_context, name);
 
-			sn_launcher_context_set_description (sn_context, description);
+				description = g_strdup_printf (_("Starting %s"), name);
 
-			g_free (description);
+				sn_launcher_context_set_description (sn_context, description);
+
+				g_free (description);
+			}
+
+			icon = mate_desktop_item_get_string (item,
+							     MATE_DESKTOP_ITEM_ICON);
+
+			if (icon != NULL)
+				sn_launcher_context_set_icon_name (sn_context, icon);
+
+			sn_launcher_context_set_workspace (sn_context, workspace);
+
+			if (startup_class != NULL)
+				sn_launcher_context_set_wmclass (sn_context,
+								 startup_class);
+		} else {
+			sn_context = NULL;
 		}
-
-		icon = mate_desktop_item_get_string (item,
-						      MATE_DESKTOP_ITEM_ICON);
-
-		if (icon != NULL)
-			sn_launcher_context_set_icon_name (sn_context, icon);
-
-		sn_launcher_context_set_workspace (sn_context, workspace);
-
-		if (startup_class != NULL)
-			sn_launcher_context_set_wmclass (sn_context,
-							 startup_class);
-	} else {
+	}
+	else {
 		sn_context = NULL;
+		sn_display = NULL;
 	}
 #endif
 
@@ -1943,7 +1946,7 @@ ditem_execute (const MateDesktopItem *item,
 		g_slist_free_full (vector_list, g_free);
 
 #ifdef HAVE_STARTUP_NOTIFICATION
-		if (sn_context != NULL &&
+		if (sn_context != NULL && (GDK_IS_X11_DISPLAY (display)) &&
 		    !sn_launcher_context_get_initiated (sn_context)) {
 			guint32 launch_time;
 
@@ -1977,7 +1980,6 @@ ditem_execute (const MateDesktopItem *item,
 		}
 #endif
 
-
 		if ( ! g_spawn_async (working_dir,
 				      real_argv,
 				      envp,
@@ -2010,7 +2012,7 @@ ditem_execute (const MateDesktopItem *item,
 
 	g_free (exec_locale);
 #ifdef HAVE_STARTUP_NOTIFICATION
-	if (sn_context != NULL) {
+	if ((sn_context != NULL) && (GDK_IS_X11_DISPLAY (display))){
 		if (ret < 0)
 			sn_launcher_context_complete (sn_context); /* end sequence */
 		else
@@ -2019,8 +2021,9 @@ ditem_execute (const MateDesktopItem *item,
 					     sn_context);
 		sn_launcher_context_unref (sn_context);
 	}
+	if (GDK_IS_X11_DISPLAY (display))
+		sn_display_unref (sn_display);
 
-	sn_display_unref (sn_display);
 #endif /* HAVE_STARTUP_NOTIFICATION */
 
 	free_args (args);
@@ -2059,7 +2062,6 @@ strip_the_amp (char *exec)
 	return TRUE;
 }
 
-
 static int
 mate_desktop_item_launch_on_screen_with_env (
 		const MateDesktopItem       *item,
@@ -2109,7 +2111,6 @@ mate_desktop_item_launch_on_screen_with_env (
 		return -1;
 	}
 
-
 	if (exec == NULL ||
 	    exec[0] == '\0') {
 		g_set_error (error,
@@ -2119,7 +2120,6 @@ mate_desktop_item_launch_on_screen_with_env (
 		return -1;
 	}
 
-
 	/* make a new copy and get rid of spaces */
 	the_exec = g_alloca (strlen (exec) + 1);
 	g_strlcpy (the_exec, exec, strlen (exec) + 1);
@@ -2436,8 +2436,6 @@ mate_desktop_item_set_entry_type (MateDesktopItem *item,
 	}
 }
 
-
-
 /**
  * mate_desktop_item_get_file_status:
  * @item: A desktop item
@@ -2468,7 +2466,7 @@ mate_desktop_item_get_file_status (const MateDesktopItem *item)
 
 	if (!g_file_info_has_attribute (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
 		retval = MATE_DESKTOP_ITEM_DISAPPEARED;
-	else if (item->mtime < g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
+	else if (((guint64) item->mtime) < g_file_info_get_attribute_uint64 (info, G_FILE_ATTRIBUTE_TIME_MODIFIED))
 		retval = MATE_DESKTOP_ITEM_CHANGED;
 
 	g_object_unref (info);
@@ -2526,7 +2524,6 @@ mate_desktop_item_find_icon (GtkIconTheme *icon_theme,
 		    *p = 0;
 		}
 
-
 		info = gtk_icon_theme_lookup_icon (icon_theme,
 						   icon_no_extension,
 						   desired_size,
@@ -3062,7 +3059,6 @@ cannonize (const char *key, const char *value)
 	return NULL;
 }
 
-
 static char *
 decode_string_and_dup (const char *s)
 {
@@ -3448,7 +3444,6 @@ insert_key (MateDesktopItem *item,
 		}
 	}
 
-
 	if (cur_section == NULL) {
 		/* only add to list if we haven't seen it before */
 		if (g_hash_table_lookup (item->main_hash, k) == NULL) {
@@ -3521,7 +3516,6 @@ try_english_key (MateDesktopItem *item, const char *key)
 	return str;
 }
 
-
 static void
 sanitize (MateDesktopItem *item, const char *uri)
 {
